export interface RawHeaders {
    "Content-Type": string;
    "Content-Length"?: string;
}
export interface FormDataEncoderHeaders extends RawHeaders {
    "content-type": string;
    "content-length"?: string;
}
